"""The Sirius Calendar Service.

This file provides a lightweight suite of slice management tools.
"""

import getpass
import time
from xmlrpclib import ServerProxy


DEBUG = False

########################################

plc = ServerProxy('https://www.planet-lab.org/PLCAPI/')

auth = dict(Username='deisenst@cs.princeton.edu',
            AuthMethod='password',
            AuthString=getpass.getpass())

print plc.AuthCheck(auth)

########################################

SLICE_URL = 'http://www.cs.princeton.edu/~deisenst/siriusdoc/'

SLICE_NAME = 'princeton_sirius'

SLICE_DESC = '''
The Sirius Calendar Service.

Sirius provides system-wide reservations of 25% CPU and 2Mb/s outgoing
bandwidth.  Sign up for hour-long slots using the Web GUI at the
PlanetLab website.

This slice should not generate traffic external to PlanetLab.
'''.lstrip()

########################################

def updatesliceattribute(sliceid, attributename, attributevalue):
    attributerecords = plc.GetSliceAttributes(auth,
                                              dict(slice_id=[sliceid],
                                                   name=attributename))
    if attributerecords:
        plc.UpdateSliceAttribute(auth,
                                 attributerecords[0]['slice_attribute_id'],
                                 str(attributevalue))
    else:
        plc.AddSliceAttribute(auth, sliceid, attributename, attributevalue)


def add():
    print 'check if the slice already exists'
    slicerecords = plc.GetSlices(auth,
                                 dict(name=SLICE_NAME),
                                 ['slice_id'])
    if slicerecords:
        sliceid = slicerecords[0]['slice_id']
    else:
        print 'make it'
        sliceid = plc.AddSlice(auth, dict(url=SLICE_URL,
                                          name=SLICE_NAME,
                                          description=SLICE_DESC))

    print 'add a user'
    plc.AddPersonToSlice(auth, auth['Username'], SLICE_NAME)

    print 'add a CPU reservation'
    updatesliceattribute(sliceid, 'cpu_min', 250)

    print 'add a bw reservation'
    updatesliceattribute(sliceid, 'net_min_rate', 2000)

    print 'add an initscript'
    newinitscript = dict(enabled=True,
                         name=SLICE_NAME,
                         script=open('sirius').read())
    oldinitscripts = plc.GetInitScripts(auth, dict(name=SLICE_NAME))
    if oldinitscripts:
        initscriptid = oldinitscripts[0]['initscript_id']
        plc.UpdateInitScript(auth, initscriptid, newinitscript)
    else:
        initscriptid = plc.AddInitScript(auth, newinitscript)

    updatesliceattribute(sliceid, 'plc_initscript_id', initscriptid)

    print 'add nodes'
    if DEBUG:
        nodefilter = ['planetlab-12.cs.princeton.edu']
    else:
        nodefilter = {}
    allnodes = plc.GetNodes(auth, nodefilter, ['node_id', 'slice_ids_whitelist'])
    allnodeids = [node['node_id'] for node in allnodes if not node['slice_ids_whitelist']]
    plc.AddSliceToNodes(auth, SLICE_NAME, allnodeids)

    print 'renew slice'
    eightweeksfromnow = int(time.time() + (8*7 - 1) * 24 * 60 * 60)
    plc.UpdateSlice(auth, SLICE_NAME, dict(expires=eightweeksfromnow))


def delete():
    plc.DeleteSlice(auth, SLICE_NAME)
